#This script is available at nickchk.com/RWorkshopScript.zip

#Comments follow hashtags
#always always always:
help()

######BASIC DATA MANIPULATION

#R is a vector-based language
#Vectors are the unit. 
#Vectors are made up of individual pieces of data of different type
a <- 1
is.numeric(a)
b <- "Hello"
is.character(b)
c <- as.factor(b)
# =  works in addition to <- if you prefer
d = 2

#R works with objects, not commands. You don't tell R to do something. You 
#manipulate objects. So in c <- as.factor(b) above, you're not saying
#"R, run the 'create variable' command, creating a variable called "c" and making it
#equal to as.factor(b)." Instead, what you're saying is
#"take the object b, and then create a factor object out of it. Then file that object under c."
#Even when running a regression, you're really "creating a regression object."

#Vectors must have entries of one type
#we can build them with the concatenate c() function
v <- c(1,2,3)
v
#and refer to them with standard indices
v[1]
v[1:2]

#Vectors are built up into lists.
#Lists are more flexible. A matrix in R is just a list of vectors
w <- c(4,5,6)
list1 <- as.matrix(cbind(v,w))
list1

#Most of the time we will be working with "data frames"
#which are matrices with the standard each-column-is-a-variable structure
df <- as.data.frame(list1)
#refer to elements using indices, or the handy $ operator
df$v
df[1,]
df[,1]
df[1,2]
df[2,'v']
df[2,1:2]
#The $ operator gives us back our vector which can come in handy
df$v[2]
#and we can use vectors when referring to elements
df[c(1,3),'w']
df[2,c('v','w')]
df[2,c(1,2)]

df$v > 1
df[(df$v > 1),]

#We can add new columns in basically the way you'd expect
df$x <- df$v + df$w
df

#the data frame stores its attributes in a way that's pretty easy to manipulate
names(df)
#it's a vector!
names(df)[3] <- "z"
df

######LIBRARIES
#A bit of a pain in my opinion. 
#Much of R is in downloadable libraries, so it's important to know
#where the best commands are stored (the internet is your friend)
#Unfortunately, packages must be re-loaded every time R starts
#which does help remind you the package they came from
#but also is a bit of a pain

install.packages('AER')
library(AER)
require(car)

#Packages we'll be discussing at least briefly today, may as well install them all now
install.packages('car')
install.packages('foreign')
install.packages('psych')
install.packages('lmtest')
install.packages('dynlm')
install.packages('tseries')
install.packages('forecast')
install.packages('plm')
install.packages('mfx')
install.packages('censReg')
install.packages('sampleSelection')

######DATA
#Data can be read from file, from the internet, or from packages

#I recommend avoiding trying to read from file, but if you do make sure
#the working directory is set first. Note that it's fiddly about Windows filepaths
setwd('C:/')
#not setwd('C:\')
#I recommend using RStudios Session -> Set Working Directory instead
#(note the handy "To Source File Location" option)

#Reading from the internet
#Plenty of data sets out there ready to be plucked from the internet
#Let's load a Wooldridge example dataset
#Note that it`s a Stata .dta file, so we need the "foreign" library to open it.
#This would not be necessary for read.csv or other native formats
library(foreign)
wage1 <- read.dta("http://fmwww.bc.edu/ec-p/data/wooldridge/wage1.dta")
#NOTE THAT IF YOU WANT TO USE STATA FILES FROM MORE RECENT VERSIONS OF STATA
#YOU WILL INSTEAD USE read.dta13 FROM THE readstata13 LIBRARY
#this is a proper data set, let's look at it
#Can click on it in the Environment window to the right
#Or the handy "head" function that gives you the first few rows
head(wage1)

#Also very good for the classroom is example data sets from packages
#note throughout that RStudio will help complete commands
library(AER)
data("ChinaIncome")

#As long as we're talking about files let's talk about exporting what we've done
#There is of course the write.csv (or write.whatever) command to save data
#there are also commands for logging output (sink)
#saving images (easy - just png(filename.png) before your image command, then dev.off() after)
#saving regression results (see the stargazer package)
#but for classroom purposes it's probably just easier to use the RStudio GUI

#I assume you're not working with missing data in the files you use for class
#but there is of course an r MICE implementation,
#or more commonly a "na.rm" or "na.omit" option when using a function
mean(wage1$wage,na.rm=TRUE)
#or you can subset on complete cases
wage1nomiss <- wage1[complete.cases(wage1),]
#and check for missing data of course
wage1$bad <- NA
table(wage1$bad)
sum(is.na(wage1$bad))

######BASIC ANALYSIS
#Most things work basically like you'd expect them to
#mean
mean(wage1$wage)
mean(log(wage1$wage))
#mean, quartiles, min, max
summary(wage1$wage)
#sd
sd(wage1$wage)
#Correlation
cor(wage1$wage,wage1$educ)
#Tabulation
table(wage1$educ)
#Crosstabulation
table(wage1$married,wage1$female)

#If you want a more detailed summary stats table you'll want describe from the psych package
library(psych)
describe(wage1$wage)
describe(wage1[,c('wage','educ','exper','tenure')])

#We can get fancy, too, or at least YOU can, for demonstration.
#Probably not teaching your students this stuff:
#Remember, if it's a data frame you can treat it like one!
describe(wage1[,c('wage','educ','exper','tenure')])[,c('n','mean')]

#Mean within groups
aggregate(wage~female,data=wage1,mean)
#Mean within crosstabulation
aggregate(wage~female+married,data=wage1,mean)
#Ooh, what if we put that in a nice table?
xtabs(wage~female+married,aggregate(wage~female+married,data=wage1,mean))

#What if we want to do a subset?
#Could use indexing as before
mean(wage1$wage[wage1$female == 1])
mean(wage1[wage1$female == 1,]$wage)
mean(wage1[wage1$female == 1 & wage1$married == 0,]$wage)
#or the subset function
head(subset(wage1,female==1))
mean(subset(wage1,female==1)$wage)
wageswomen <- subset(wage1,female==1)
mean(wageswomen$wage)

######BASIC PLOTS
#R's plot function is pretty darn magical. It knows what you want most of the time.
#ChinaIncome is declared as a timeseries variable, so it plots a timeseries
plot(ChinaIncome)
plot(ChinaIncome[,5])
plot(ChinaIncome[,5],ylab="Transport")
#R plots can be built over several lines. Many commands, like title, can be added
#after the initial plot is run
title("Chinese Transport Income over Time")

plot(wage1$tenure,log(wage1$wage))
#abline lets you put a line on there with intercept and slope
abline(0,1)

#Abline is flexible! Want to plot a regression line? Just stick 
#a regression "object" in abline
reg <- lm(log(wage)~tenure,data=wage1)
plot(wage1$tenure,log(wage1$wage))
abline(reg)

#plot can also give us a density disribution
plot(density(wage1$wage))

#although sometimes we want a histogram
hist(wage1$wage)

#Want to get fancy? You can add a curved line too.
#let's add a normal dist on top of our histogram
hist(wage1$wage,prob=TRUE)
curve(dnorm(x,mean(wage1$wage),sd(wage1$wage)),from=0,to=25,add=TRUE)
#unlike abline, curve needs an "add=TRUE" option to draw over existing graphs

#Can write our own functions too
plot(wage1$tenure,log(wage1$wage))
fun <- function(x) log(x)
curve(fun,0,45,add=TRUE)

#Barplot is pretty useful too, and flexible
#a little harder to use. See help(barplot)
barplot(table(wage1$married,wage1$female))

#All of these graphs have LOTS of options, see the help files if you want to do anything fancy and it WILL be there.

#Note that if you want to get real serious into R graphing
#(and it does get very pretty)
#look into learning the ggplot2 package
#although teaching this to students would probably be too difficult

######REGRESSION
#In R it's helpful to think of a regression not as a command but as an OBJECT
#with properties
#the "lm" command does OLS for us
#if we just run lm it doesn't give us a whole lot
lm(wage~educ+female+married+tenure,data=wage1)
#because we're supposed to store it as an object and then use it!
myreg <- lm(wage~educ+female+married+tenure,data=wage1)
#Let's look at the object
summary(myreg)
#NOTE that if you want to output the results in a very pretty way
#see the stargazer package
#It prints nicer, and also you can output a nice formatted table to file
install.packages('stargazer')
library(stargazer)
stargazer(myreg,type="text")

#We can get residuals from this object
residuals <- resid(myreg)
plot(wage1$educ,residuals)
abline(0,0)

#and predicted values
fitted <- predict(myreg)
plot(wage1$educ,fitted)
abline(myreg)

#Several useful postestimation commands are in lmtest
library(lmtest)

#like robust errors, although honestly a bit of a pain
coeftest(myreg,vcov=sandwich)
#note that hccm is available instead of sandwich if you prefer

#also in lmtest is Breusch-Pagan
bptest(myreg)

#If you want to do a joint F test, use linearHypothesis from car
#(companion to applied regression)
library(car)
linearHypothesis(myreg,c("educ","female=married"))
#can also perform the test with robust errors
linearHypothesis(myreg,c("educ","female=2*married"),vcov=sandwich)

#R is very flexible in terms of how you can write the formula
#Unlike some packages, you can do logs, interactions, etc, right in the formula
#Turn continuous variable into a dummy with I()
summary(lm(wage~educ+female+married+I(tenure>5),data=wage1))
#or do on-the-fly variable manipulation with I()
summary(lm(wage~educ+female+married+I(tenure^2),data=wage1))
#Interaction plus the variables on their own
summary(lm(wage~educ+female*married+tenure,data=wage1))
#interaction only
summary(lm(wage~educ+female:married+I(tenure>5),data=wage1))
#why the heck are we using wage? Shouldn't that be logged?
summary(lm(log(wage)~educ+female+married+tenure+I(tenure^2),data=wage1))
#Maybe we should be doing education as dummies using factor
summary(lm(log(wage)~factor(educ)+female+married+tenure+I(tenure^2),data=wage1))
#(factor is another data type in addition to numeric, character, etc.)


#Of course, we don't necessarily need to jump to regression. t-tests are good too
#Is the average wage 5?
t.test(wage1$wage-5)
#is the wage different for men and women?
t.test(wage1$wage~wage1$female)

#How about if we want IV? It's in AER.
library(AER)
#don't forget to put covariates in both stages. stage 2 | stage 1
ivmodel <- ivreg(wage~tenure+female|educ+female,data=wage1)
summary(ivmodel)
#how about some weak-instrument tests (F test on first stage instruments)? Sargan? Wu-Hausman?
summary(ivmodel,diagnostics=TRUE)
#Unfortunately it won't show you the first stage automatically, have to do it by hand
firststage <- lm(tenure~educ+female,data=wage1)
summary(firststage)

######TIME SERIES
#Here we'll be working with dynlm, which gives us access to lags
library(dynlm)
industry <- ChinaIncome[,5]
summary(dynlm(industry~L(industry,1:2)))

#and tseries, which gives us all sorts of cool stuff
library(tseries)
#see more about this package at https://www.statmethods.net/advstats/timeseries.html 

#Have a vector that's time series but R doesn't know it? Declare the thing with ts()
#let's pretend wage1 is monthly data starting in 1990
fakets <- ts(wage1$wage,start=c(1990,1),frequency=12)
plot(fakets)
#or we can make it end today
fakets <- ts(wage1$wage,end=c(2018,6),frequency=12)
plot(fakets)

#Now let's go back to our real time series
#like ACFs
acf(industry)
#and pacfs
pacf(industry)

#Dickey-fuller
adf.test(industry)

#Let's tell R that industry follows a 12-month cycle and then do seasonal analysis
industry <- ts(industry,frequency=12)
seasons <- stl(industry,s.window="period")
plot(seasons)

#Or from the forecast package, seasonplot which overlays the years
library(forecast)
seasonplot(industry)

#do you guys teach ARIMA to undergrads?
ar111 <- arima(industry,order=c(1,1,1))
summary(ar111)
#and from the forecast package, we can forecast the next obs
forecast(ar111,5)
plot(forecast(ar111,5))

######PANEL DATA
#let's load in a panel data set
#just make sure we have foreign loaded
library(foreign)
crime <- read.dta("http://fmwww.bc.edu/ec-p/data/wooldridge/crime4.dta")

#Now to work with panel data we'll load the panel data model package
library(plm)
#and tell R that we have panel data
crime.p <- pdata.frame(crime,index=c("county","year"))

#plm gives us the models we want. (fd)irstdiff, (w)ithin, (b)etween, (r)andom
firstd <- plm(log(crmrte)~polpc,data=crime.p,model="fd")
summary(firstd)
fixedeffects <- plm(log(crmrte)~polpc,data=crime.p,model="w")
summary(fixedeffects)
betweenmodel <- plm(log(crmrte)~polpc,data=crime.p,model="b")
summary(betweenmodel)
randomeffects <- plm(log(crmrte)~polpc,data=crime.p,model="r")
summary(randomeffects)

#how about a lag or a difference?
mymodel <- plm(log(crmrte)~lag(polpc,1)+diff(taxpc,1),data=crime.p,model="w")
summary(mymodel)

#The allure of the Hausman test eludes me but here it is!
phtest(fixedeffects,randomeffects)

#How about clustered errors? Like with robust before, we get the model
#object first and adjust the errors after
library(lmtest)
coeftest(fixedeffects,vcovHC)

######LIMITED DEPENDENT VARIABLES AND SELECTION
#We're back to wage1! Logit and probit are run using GLM
logitmarried <- glm(married~female+tenure,family=binomial(link=logit),data=wage1)
summary(logitmarried)
probitmarried <- glm(married~female+tenure,family=binomial(link=probit),data=wage1)
summary(probitmarried)

#If we want marginal effects we will need the mfx package
library(mfx)
#We can get marginal effects using the logitmfx and probitmfx functions, respectively
#use atmean = TRUE or FALSE to get MEM and AME, as appropriate
logitmfx(logitmarried,atmean=TRUE,data=wage1)
probitmfx(probitmarried,atmean=FALSE,data=wage1)

#tobit can be found in the package censReg
library(censReg)
#Let's get some censoring in there
wagecensor <- wage1
wagecensor$wage[wagecensor$wage < 3] <- NA
sum(is.na(wagecensor$wage))

tobitmodel <- censReg(wage~female+married+tenure,data=wagecensor,left=3)
summary(tobitmodel)
#Marginal effects come from the margEff command, in the same package
margEff(tobitmodel)

#Heckman, not too surprisingly, is in the sampleSelection package
library(sampleSelection)
heckmanmodel <- selection(!is.na(wage)~educ+female+married+tenure,wage ~ educ+female+married+tenure,data=wagecensor)
summary(heckmanmodel)


######SUPER COOL AND USEFUL STUFF THAT YOU MIGHT USE TO MAKE STUFF FOR CLASS
#BUT PROBABLY WON'T TEACH IN CLASS

#Mostly, look around online for more detail on how to use these

#ggplot2!
install.packages('ggplot2')
library(ggplot2)
#The nice and fancy way to make plots in R. These look *good* are are 
#highly customizable. More of a learning curve though.
#This command goes deep. There are books just on this command.
#A very simple example where I put the scatterplot of data in front of a barplot of means
ggplot(data = wage1) +
  geom_bar(aes(x=tenure,y=log(wage)),stat="summary",fun.y=mean) +
  geom_point(aes(x=tenure,y=log(wage)))
  

#Make maps! Also using ggplot most of the time
install.packages('maps')
install.packages('ggmaps')
install.packages('mapdata')
library(maps)
library(mapdata)
states <- map_data("state")
#Let's just basically make a state map colored with random colors.
#But of course you can make heat maps, scatterplots based on lat/lon, etc.
ggplot(data = states) + 
  geom_polygon(aes(x = long, y = lat, fill = region, group = group), color = "white") + 
  coord_fixed(1.3) +
  guides(fill=FALSE)  # do this to leave off the color legend
#see also ggmaps for more advanced stuff

#stargazer
#mentioned above. Use to export nice-looking regression tables
install.packages('stargazer')
library(stargazer)
simplereg <- lm(log(wage)~tenure,data=wage1)
stargazer(simplereg,type="text")
stargazer(simplereg,type="html",out="regression_stored.html")

#dplyr
#This one is more for if you're manipulating data to actualy work on it
#if you're familiar with Stata, this basically lets you write 
#custom egen commands on the fly with the various aply commands
#This is also very flexible and very deep
#this is a whole family of functions, ddply is only one of them
#the first two characters tell you what the in and out of the function are
install.packages('dplyr')
library(dplyr)
install.packages('plyr')
library(plyr)
#add a column with average education by number of dependents and marriage
wage1 <- ddply(wage1,.(numdep,married),transform, edmean = mean(educ))
